#!zsh
# git-supertree completion script
#
# Completion script documentation:
# https://github.com/zsh-users/zsh-completions/blob/master/zsh-completions-howto.org

_git-supertree()
{
	local curcontext="$curcontext" state line
	typeset -A opt_args

	_arguments -C \
		':command:->command' \
		'*::options:->options'

	case $state in
		(command)

			local -a subcommands
			subcommands=(
				'init:Initialize a new git repo with support for the supertree layout .'
				'clone:Clone a new git repo with support for the supertree layout.'
				'convert:Convert a vanilla repo to a supertree layout.'
				'add:Add a worktree to the supertree.'
				'rm:Remove a worktree.'
			)
			_describe -t commands 'git supertree' subcommands
		;;

		(options)
			case $line[1] in

				(init)
					_arguments \
						'--branch[Initial branch and worktree name]:BRANCH:__git_supertree_branch_spec' \
						'::path:_directories' \
					;;

				(clone)
					_arguments \
						'--branch[Initial branch and worktree name]:BRANCH:__git_supertree_branch_spec' \
						':repository_url:__git_any_repositories' \
						':repository_url:_directories' \
						'::path:_directories' \
					;;

				(convert)
					declare -a refs=( $(git for-each-ref \
						--format='%(refname:short)' \
						refs/heads \
					) )
					_arguments \
						"--master[Override the value of the base HEAD symbolic ref]:ref:(${refs[*]})" \
						'::path:_directories' \
					;;

				(add)
					declare -a refs=( $(git for-each-ref \
						--format='%(refname:short)' \
						refs/heads \
						refs/remotes \
					) )
					_arguments \
						"--from[Use a branch other than the value of the base HEAD symbolic ref]:ref:(${refs[*]})" \
						':BRANCH:__git_supertree_branch_spec' \
					;;

				(rm)
					_arguments \
						'--rm-branch[Delete the branch as well as the working tree]' \
						':path:_directories' \
					;;
			esac
		;;
	esac
}

__git_supertree_branch_spec()
{
	declare -a refs=( $(git for-each-ref --format='%(refname:short)' refs/heads) )
	refs+=( $(git for-each-ref --format='%(refname:strip=3)' refs/remotes) )
	_sep_parts "(${refs[*]})" : '*'
}

zstyle ':completion:*:*:git:*' user-commands supertree:'manage a collection of worktrees'
